//----------------------------------------------------------------
//	Test Simple Interrupt using C
//
//	dependency: none
//	2001, 11, 1
//	Author: Nelson Y.C. Chang
//
//----------------------------------------------------------------

#include <stdio.h>




unsigned Install_Handler( unsigned routine, unsigned *vector )
{
	unsigned vec, oldvec;
	vec = ((routine - (unsigned)vector - 0x8) >> 2 ); /* --> routine is the pointer point to the IRQ handler. */
							  /* --> shift right 2 is for address word aligned. */
							  /* --> subtract 8 is due to the pipeline */
							  /* since PC will be fetching the 2nd instruction */
							  /* after the instruction currently being executed. */	
	vec = 0xea000000 | vec; /* to implement the instruction B <address> */
				/* 0xea is the Branch operation */
	oldvec = *vector; 	/* the IRQ address or FIQ address */
	*vector = vec; 		/* the contents of IRQ address is now the branch instruction */
	return (oldvec); 
}

// this function is used to set the I bit in CPSR
__inline void enable_IRQ(void) 
{
  int tmp;
  __asm
  {
    MRS tmp, CPSR
    BIC tmp, tmp, #0x80
    MSR CPSR_c, tmp
  }
}

//------------------------------------------
//	Timer related
//-------------------------------------------

void LoadTimer(int loadvalue){
	int TIMER0_LOAD_ADDR = 0x13000000;
	int *TIMER0_LOAD;
	
	TIMER0_LOAD = (int *)TIMER0_LOAD_ADDR;
	
	*TIMER0_LOAD = loadvalue;
	printf("Timer Message>>> Timer0 loaded!!\n");
}


int ReadTimer(void){
	int TIMER0_VALUE_ADDR = 0x13000004;
	int *TIMER0_VALUE;
	
	TIMER0_VALUE = (int *)TIMER0_VALUE_ADDR;

	printf("Timer Message>>> Timer0 value aquired!!\n");
	return *TIMER0_VALUE;
}


void WriteTimerCtrl(int writevalue){
	int TIMER0_CTRL_ADDR = 0x13000008;
	int *TIMER0_CTRL;
	
	TIMER0_CTRL = (int *)TIMER0_CTRL_ADDR;
	
	*TIMER0_CTRL = writevalue;
	printf("Timer Message>>> Timer0 control register changed!!\n");
}

void ClearTimer(void){
	int TIMER0_CLEAR_ADDR = 0x1300000C;
	int *TIMER0_CLEAR;
	
	TIMER0_CLEAR = (int *)TIMER0_CLEAR_ADDR;
	
	*TIMER0_CLEAR = 1;
	printf("Timer Message>>> Timer0 cleared!!\n");
}

// IRQ Handler for timer0
__irq void myIRQHandler (void)
{
	printf("\n myInterruptHandler>> HIHI!!\n");
	ClearTimer();
}

//================================================
void d2b(int d_number, int array_len, int *b_number) {
 int len; /*array index*/
 int temp=1;

 for (len=0;len<array_len;len++) {
     if (temp&d_number) b_number[len]=1;
     else b_number[len]=0;
     d_number=d_number>>1;
 }
}

void printB(int d_number, int array_len, int*b_number){
	int i;
	for(i=(array_len-1);i>=0;i--){
		printf("%d",b_number[i]);
	   	if ( i%8==0 && i!=0)
     	printf("_");
	}
	printf("\n");
}


int main(void) {

	int IRQ0_STATUS_ADDR = 0x14000000;
	int IRQ0_RAWSTAT_ADDR = 0x14000004;
	int IRQ0_ENABLESET_ADDR = 0x14000008;
	int IRQ0_ENABLECLR_ADDR = 0x1400000C;
	
	int *IRQ0_STATUS, *IRQ0_RAWSTAT, *IRQ0_ENABLESET, *IRQ0_ENABLECLR;
	
	int b_num[22];
	int i;
	unsigned *irqvec = (unsigned *)0x18;
	WriteTimerCtrl(0xC4);			// disable the timer first.
		
	Install_Handler( (unsigned)myIRQHandler, irqvec );
	
	/* DR added - ENABLE IRQs */
	enable_IRQ();
	
	
	IRQ0_STATUS = (int *) IRQ0_STATUS_ADDR;
	IRQ0_RAWSTAT = (int *) IRQ0_RAWSTAT_ADDR;
	IRQ0_ENABLESET = (int *) IRQ0_ENABLESET_ADDR;
	IRQ0_ENABLECLR = (int *) IRQ0_ENABLECLR_ADDR;	

	*IRQ0_ENABLESET = 0x0021;

	// print value of IRQ0_STATUS
	d2b(*IRQ0_STATUS,22,b_num);
	printf("IRQ0_STATUS: ");
	printB(*IRQ0_STATUS,22,b_num);

	// print value of IRQ0_RAWSTATUS
	d2b(*IRQ0_RAWSTAT,22,b_num);
	printf("IRQ0_RAWSTAT: ");
	printB(*IRQ0_RAWSTAT,22,b_num);

	// print value of IRQ0_ENABLESET
	d2b(*IRQ0_ENABLESET,22,b_num);
	printf("IRQ0_ENABLESET: ");
	printB(*IRQ0_ENABLESET,22,b_num);

	// print value of IRQ0_ENABLCLR
	d2b(*IRQ0_ENABLECLR,22,b_num);
	printf("IRQ0_ENABLECLR: ");
	printB(*IRQ0_ENABLECLR,22,b_num);

	//==============================

	LoadTimer(64);				// load timer value
	WriteTimerCtrl(0xC4);		// enable the timer
	// wait for a while
	for(i=0;i<1000000000;i++)		
	{
//		if((i%100000) == 99999)
//			printf("#");
	;
	}
	printf("\nEND\n");
	return 0;
}